package com.plannaplan.controllers;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.annotation.DirtiesContext.MethodMode;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import static org.springframework.security.test.web.servlet.setup.SecurityMockMvcConfigurers.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.nio.charset.Charset;

import com.plannaplan.entities.Assignment;
import com.plannaplan.entities.Commision;
import com.plannaplan.entities.Course;
import com.plannaplan.entities.Exchange;
import com.plannaplan.entities.Groups;
import com.plannaplan.entities.User;
import com.plannaplan.services.AssignmentService;
import com.plannaplan.services.CommisionService;
import com.plannaplan.services.CourseService;
import com.plannaplan.services.ExchangeService;
import com.plannaplan.services.GroupService;
import com.plannaplan.services.UserService;
import com.plannaplan.types.UserRoles;
import com.plannaplan.types.WeekDay;

@RunWith(SpringRunner.class)
@SpringBootTest
@ContextConfiguration
public class ExchangeControllerTest extends AbstractControllerTest {

    private final static String EXCHANGE_ENDPOINT = "/api/v1/exchanges/exchange";
    private static final MediaType APPLICATION_JSON_UTF8 = new MediaType(MediaType.APPLICATION_JSON.getType(),
            MediaType.APPLICATION_JSON.getSubtype(), Charset.forName("utf8"));

    @Autowired
    private UserService userService;

    @Autowired
    private GroupService groupService;

    @Autowired
    private AssignmentService assignmentService;

    @Autowired
    private CommisionService commisionService;

    @Autowired
    private ExchangeService exchangeService;

    @Autowired
    private CourseService courseService;

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldGetAllUsersExchanges() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldGetAllUsersExchanges@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        this.exchangeService.save(new Exchange(assignment, groupDesired));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(EXCHANGE_ENDPOINT + "/all").header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());
    }

    @Test
    public void shouldFailGettingNotExistingExchange() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailGettingNotExistingExchange@ExchangeController.test", UserRoles.STUDENT));

        final String token = this.userService.login(user).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(EXCHANGE_ENDPOINT + "/" + user.getId()).header("Authorization", "Bearer " + token))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldGetSingleExchange() throws Exception {
        final User user = this.userService.save(new User(null, null, "shouldGetSingleExchange@ExchangeController.test",
                "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        final Exchange exchange = this.exchangeService.save(new Exchange(assignment, groupDesired));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(EXCHANGE_ENDPOINT + "/" + exchange.getId()).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());
    }

    @Test
    public void shouldFailGettingExchangeDueToPermission() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailGettingExchangeDueToPermission@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        final Exchange exchange = this.exchangeService.save(new Exchange(assignment, groupDesired));

        final User user2 = this.userService.save(new User(null, null,
                "shouldFailGettingExchangeDueToPermission2@ExchangeController.test", "11112", UserRoles.STUDENT, 321));
        final String token2 = this.userService.login(user2).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT + "/" + exchange.getId()).header("Authorization", "Bearer " + token2))
                .andExpect(status().is4xxClientError());
    }

    @Test
    public void shouldFailPostDueToAssignmentNotFound() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailPostDueToAssignmentNotFound@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(212, "A2-1", null, 420, WeekDay.WEDNESDAY, null));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(
                post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token).contentType(APPLICATION_JSON_UTF8)
                        .content("{\"assignment\": " + user.getId() + ", \"group\": " + group.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldInsertExchange() throws Exception {
        final User user = this.userService.save(
                new User(null, null, "shouldInsertExchange@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));

        this.assignmentService.callAcceptAlgorythm();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token)
                .contentType(APPLICATION_JSON_UTF8)
                .content("{\"assignment\": " + assignment.getId() + ", \"group\": " + groupDesired.getId() + " }"))
                .andExpect(status().isOk());
    }

    @Test
    public void shouldFailInsertExchangeDueToMissingGroup() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailInsertExchangeDueToMissingGroup@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token)
                .contentType(APPLICATION_JSON_UTF8).content("{\"assignment\": " + user.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    public void shouldFailInsertExchangeDueToMissingAssignment() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailInsertExchangeDueToMissingAssignment@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(212, "A2-1", null, 420, WeekDay.WEDNESDAY, null));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token)
                .contentType(APPLICATION_JSON_UTF8).content("{\"group\": " + group.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    public void shouldFailInsertExchangeDueToMissingParam() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailInsertExchangeDueToMissingParam@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(
                post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token).contentType(APPLICATION_JSON_UTF8))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldDenyExchangeDueToAssigmentOverlapping() throws Exception {
        final User user = this.userService
                .save(new User(null, null, "shouldDenyExchangeDueToAssigmentOverlapping@ExchangeController.test",
                        "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        this.exchangeService.save(new Exchange(assignment, group));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(
                post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token).contentType(APPLICATION_JSON_UTF8)
                        .content("{\"assignment\": " + user.getId() + ", \"group\": " + group.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    public void shouldDenyPostDueToAssignmentNotAccepted() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldDenyPostDueToAssignmentNotAccepted@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(212, "A2-1", null, 420, WeekDay.WEDNESDAY, null));
        final Groups group2 = this.groupService.save(new Groups(213, "A2-2", null, 420, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(
                post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token).contentType(APPLICATION_JSON_UTF8)
                        .content("{\"assignment\": " + assignment.getId() + ", \"group\": " + group2.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldDeleteExchange() throws Exception {
        final User user = this.userService.save(
                new User(null, null, "shouldDeleteExchange@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        final Exchange exchange = this.exchangeService.save(new Exchange(assignment, groupDesired));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(delete(EXCHANGE_ENDPOINT + "/" + exchange.getId()).header("Authorization", "Bearer " + token))
                .andExpect(status().isOk());
    }

    @Test
    public void shouldFailDeleteDueToWrongPermissions() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailDeleteDueToWrongPermissions@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        final Exchange exchange = this.exchangeService.save(new Exchange(assignment, groupDesired));

        final User user2 = this.userService.save(new User(null, null,
                "shouldFailDeleteDueToWrongPermissions@ExchangeController2.test", "11112", UserRoles.STUDENT, 322));
        final String token2 = this.userService.login(user2).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(get(EXCHANGE_ENDPOINT + "/" + exchange.getId()).header("Authorization", "Bearer " + token2))
                .andExpect(status().isBadRequest());
    }

    @Test
    public void shouldFailDeleteDueToMissingParam() throws Exception {
        final User user = this.userService.save(
                new User(null, null, "shouldFailDeleteDueToMissingParam@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(delete(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token))
                .andExpect(status().is4xxClientError());
    }

    @Test
    public void shouldFailDeleteDueToExchangeNotFound() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailDeleteDueToExchangeNotFound@ExchangeController.test", UserRoles.STUDENT));
        final String token = this.userService.login(user).getToken();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(delete(EXCHANGE_ENDPOINT + "/" + user.getId()).header("Authorization", "Bearer " + token))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldFailPostDueToGroupAlreadyAccepted() throws Exception {
        final User user = this.userService.save(new User(null, null,
                "shouldFailPostDueToGroupAlreadyAccepted@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Groups group = this.groupService.save(new Groups(215, "A2-2", null, 520, WeekDay.TUESDAY, null));
        final Groups group2 = this.groupService.save(new Groups(216, "A2-3", null, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));
        this.assignmentService.save(new Assignment(group2, commision));
        this.assignmentService.callAcceptAlgorythm();
        this.exchangeService.save(new Exchange(assignment, group2));

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(
                post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token).contentType(APPLICATION_JSON_UTF8)
                        .content("{\"assignment\": " + assignment.getId() + ", \"group\": " + group2.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldFailForDiffrentCoursesExchange() throws Exception {
        final User user = this.userService.save(
                new User(null, null, "shouldInsertExchange@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Course course = this.courseService
                .save(new Course("shouldFailForDiffrentCoursesExchangeCourse", "SFFDCEC"));
        final Course course2 = this.courseService
                .save(new Course("shouldFailForDiffrentCoursesExchangeCourse2", "SFFDCEC-2"));
        final Groups group = this.groupService.save(new Groups(215, "A2-2", course, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", course2, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));

        this.assignmentService.callAcceptAlgorythm();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token)
                .contentType(APPLICATION_JSON_UTF8)
                .content("{\"assignment\": " + assignment.getId() + ", \"group\": " + groupDesired.getId() + " }"))
                .andExpect(status().isBadRequest());
    }

    @Test
    @DirtiesContext(methodMode = MethodMode.BEFORE_METHOD)
    public void shouldFailForDiffrentGroupTypesExchange() throws Exception {
        final User user = this.userService.save(
                new User(null, null, "shouldInsertExchange@ExchangeController.test", "11111", UserRoles.STUDENT, 320));
        final String token = this.userService.login(user).getToken();
        final Course course = this.courseService
                .save(new Course("shouldFailForDiffrentCoursesExchangeCourse", "SFFDCEC"));
        final Groups group = this.groupService.save(new Groups(12, "A2-2", course, 520, WeekDay.TUESDAY, null));
        final Groups groupDesired = this.groupService.save(new Groups(216, "A2-3", course, 530, WeekDay.MONDAY, null));
        final Commision commision = this.commisionService.save(new Commision(user));
        final Assignment assignment = this.assignmentService.save(new Assignment(group, commision));

        this.assignmentService.callAcceptAlgorythm();

        MockMvc mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).apply(springSecurity()).build();
        mockMvc.perform(post(EXCHANGE_ENDPOINT).header("Authorization", "Bearer " + token)
                .contentType(APPLICATION_JSON_UTF8)
                .content("{\"assignment\": " + assignment.getId() + ", \"group\": " + groupDesired.getId() + " }"))
                .andExpect(status().isBadRequest());
    }
}
